--
-- Digits
-- A moveable HUD for current Digits total while in combat
-- 2008 Satrina@StormDigits
--
-- Localisation is at the end of this file
--
DigitsDB = {}

local _sml = LibStub:GetLibrary("LibSharedMedia-3.0")
local _sdd = LibStub:GetLibrary('LibScrollingDropDown-1.0')
local _lcf = LibStub:GetLibrary('LibColourFrame-1.0')
local _UnitPower = UnitPower
local _UnitPowerMax = UnitPowerMax
local _UnitHealth = UnitHealth
local _UnitHealthMax = UnitHealthMax

local DigitsFrame = CreateFrame("Frame", "DigitsFrame", UIParent)
DigitsFrame.version = 1.0
DigitsFrame.minor = 4

function DigitsFrame:Initialise()
  self:DoSavedVars()
  SLASH_DIGITS1 = "/digits"
  SlashCmdList["DIGITS"] = function() InterfaceOptionsFrame_OpenToCategory(DigitsOptionsFrame) end

  self:InitOptions()
  self:CreateFrame("DigitsPowerFrame", self.db.powerFrame)
  self:CreateFrame("DigitsHealthFrame", self.db.healthFrame)
  DigitsHealthFrame:SetFrameStrata("LOW")
  DigitsPowerFrame:SetFrameStrata("LOW")
  
  self.inCombat = false
  self:SetScript("OnUpdate", self.OnUpdate)
  self:RegisterEvent("PLAYER_ENTERING_WORLD")
  self:RegisterEvent("PLAYER_LEAVE_COMBAT")
  self:RegisterEvent("PLAYER_ENTER_COMBAT")
  self:RegisterEvent("PLAYER_REGEN_DISABLED")
  self:RegisterEvent("PLAYER_REGEN_ENABLED")
  self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
  self:RegisterEvent("UNIT_HEALTH")
  self:RegisterEvent("UNIT_POWER")
end

local e = 0
function DigitsFrame:OnUpdate(elapsed)
  e = e + elapsed
  if (e > 0.25) then
    e = 0
    if self.inCombat then
      self:UpdateHealth()
      self:UpdatePower()
    end
  end
end

function DigitsFrame:OnEvent(event)
  if (event == "VARIABLES_LOADED") then
    self:Initialise()
  elseif not self.inCombat and (((event == "UNIT_POWER") or (event == "UNIT_HEALTH")) and (arg1 == "player") and not self.showingOptions) then
    if self.initialised then
      if self.db.healthFrame.enabled then
        self:UpdateHealth()
      end
      if self.db.powerFrame.enabled and self.db[UnitPowerType("player")].enabled then
        self:UpdatePower()
      end
    end
  elseif (event == "PLAYER_ENTERING_WORLD") then
    DigitsFrame:EnablePowerFrame()
  elseif (event == "UPDATE_SHAPESHIFT_FORM") then
    DigitsFrame:EnablePowerFrame()
  elseif (event == "PLAYER_REGEN_DISABLED") then
    self.inCombat = true
  elseif (event == "PLAYER_REGEN_ENABLED") then
    self.inCombat = false
  end
  if self.initialised and not self.showingOptions then
    self:CheckCombat()
  end
end

function DigitsFrame:CreateFrame(name, var)
  local f = CreateFrame("Frame", name, UIParent)
  f.var = var
  f:SetClampedToScreen(true)
  f:SetMovable(true)
  f:SetWidth(150)
  f:SetHeight(25)
  f:SetPoint(f.var.relative, UIParent, f.var.relative, f.var.x, f.var.y)
  f:SetScript("OnDragStart", function(self) self:StartMoving() end)
  f:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() self.var.relative, _, _, self.var.x, self.var.y = self:GetPoint() end)
  f.text = f:CreateFontString()
  f.text:SetWidth(150)
  f.text:SetHeight(25)
  f.text:SetPoint("CENTER", f, "CENTER")
end

DigitsFrame.powerIndex = {0,1,2,3,6}
function DigitsFrame:DoSavedVars()
  _,self.class = UnitClass("player")
  self.player = format("%s of %s", UnitName("player"), GetRealmName())
  if not DigitsDB[self.player] then
    local colour
    ChatFrame1:AddMessage(format(self.strings.createProfile, self.player))
    DigitsDB[self.player] = {}
  end
  
  for _,i in ipairs(DigitsFrame.powerIndex) do
    if not DigitsDB[self.player][i] then
      DigitsDB[self.player][i] = {}
      DigitsDB[self.player][i].enabled = (i == 0 and ((self.class ~= "WARRIOR") and (self.class ~= "ROGUE") and (self.class ~= "DEATHKNIGHT")))
        or (i == 1 and ((self.class == "WARRIOR") or (self.class == "DRUID")))
        or (i == 2 and ((self.class == "HUNTER")))
        or (i == 3 and ((self.class == "ROGUE") or (self.class == "DRUID")))
        or (i == 6 and ((self.class == "DEATHKNIGHT")))
      colour = DigitsDB[self.player][i].enabled and PowerBarColor[i] or GRAY_FONT_COLOR
      DigitsDB[self.player][i].r = colour.r
      DigitsDB[self.player][i].g = colour.g
      DigitsDB[self.player][i].b = colour.b
      DigitsDB[self.player][i].a = 0.6
    end
  end

  if not DigitsDB[self.player][10] then
    DigitsDB[self.player][10] = {enabled = true, r = 0.1, g = 1.0, b = 0.1, a = 0.6 }
    DigitsDB[self.player].powerFrame = {}
    DigitsDB[self.player].healthFrame = {}
    DigitsDB[self.player].powerFrame.enabled = true
    DigitsDB[self.player].healthFrame.enabled = true
  end
  
  self.db = DigitsDB[self.player]
  
  self.db.powerFrame = self.db.powerFrame or {}
  self.db.powerFrame.fontSize = self.db.powerFrame.fontSize or 35
  if (self.db.powerFrame.fontSize > 35) then
    self.db.powerFrame.fontSize = 35
  end
  self.db.powerFrame.font = self.db.powerFrame.font or "Friz Quadrata TT"
  self.db.powerFrame.relative = self.db.powerFrame.relative or "CENTER"
  self.db.powerFrame.x = self.db.powerFrame.x or 100
  self.db.powerFrame.y = self.db.powerFrame.y or -250
  
  self.db.healthFrame = self.db.healthFrame or {}
  self.db.healthFrame.fontSize = self.db.healthFrame.fontSize or 35
  if (self.db.healthFrame.fontSize > 35) then
    self.db.healthFrame.fontSize = 35
  end
  self.db.healthFrame.font = self.db.healthFrame.font or "Friz Quadrata TT"
  self.db.healthFrame.relative = self.db.healthFrame.relative or "CENTER"
  self.db.healthFrame.x = self.db.healthFrame.x or -100
  self.db.healthFrame.y = self.db.healthFrame.y or -250
end

function DigitsFrame:Setup()
  DigitsHealthFrame.text:SetFont(_sml:Fetch("font", self.db.healthFrame.font), self.db.healthFrame.fontSize, self.db.healthFrame.outline)
  DigitsHealthFrame.text:SetVertexColor(self.db[10].r, self.db[10].g, self.db[10].b)
  if self.db.healthFrame.enabled then
    DigitsHealthFrame:Show()
    DigitsHealthFrame:SetAlpha(self.db[10].a)
  else
    DigitsHealthFrame:Hide()
  end

  DigitsPowerFrame.text:SetFont(_sml:Fetch("font", self.db.powerFrame.font), self.db.powerFrame.fontSize, self.db.powerFrame.outline)
  DigitsPowerFrame.text:SetVertexColor(self.db[self.powerType].r, self.db[self.powerType].g, self.db[self.powerType].b)
  if self.db.powerFrame.enabled and self.db[UnitPowerType("player")].enabled then
    DigitsPowerFrame:Show()
    DigitsPowerFrame:SetAlpha(self.db[self.powerType].a)
  else
    DigitsPowerFrame:Hide()
  end
end

function DigitsFrame:EnablePowerFrame()
  if UnitPowerType("player") ~= self.powerType then
    self.powerType = UnitPowerType("player")
    self:Setup()
    self.initialised = true
  end
end

function DigitsFrame:UpdatePower()
  local p = (self.showingOptions and _UnitPowerMax("player")) or _UnitPower("player")
  if self.db.powerFrame.percent then
    DigitsPowerFrame.text:SetFormattedText("%d%%", floor(p/_UnitPowerMax("player")*100))
  else
    DigitsPowerFrame.text:SetFormattedText("%d", p)
  end
end

function DigitsFrame:UpdateHealth()
  local h = (self.showingOptions and _UnitHealthMax("player")) or _UnitHealth("player")
  if self.db.healthFrame.percent then
    DigitsHealthFrame.text:SetFormattedText("%d%%", floor(h/_UnitHealthMax("player")*100))
  else
    DigitsHealthFrame.text:SetFormattedText("%d", h)
  end
end

function DigitsFrame:CheckCombat()
  if not self.initialised then
    return
  end
  if self.db.powerFrame.ooc or (self.db.powerFrame.enabled and self.db[UnitPowerType("player")].enabled and UnitAffectingCombat("player")) then
    DigitsPowerFrame:Show()
    DigitsFrame:UpdatePower()
  else
    DigitsPowerFrame:Hide()
  end
  if self.db.healthFrame.ooc or (self.db.healthFrame.enabled and UnitAffectingCombat("player")) then
    DigitsHealthFrame:Show()
    DigitsFrame:UpdateHealth()
  else
    DigitsHealthFrame:Hide()
  end
end



--
-- Options
--

function DigitsFrame:InitOptions()
  self.currentColour = 10
  
  DigitsOptionsFrame.name = select(2, GetAddOnInfo("Digits"))
  InterfaceOptions_AddCategory(DigitsOptionsFrame) 
  
  DigitsOptionsFrameVersion:SetFormattedText(self.strings.version, self.version, self.minor)
  DigitsOptionsFrameHealth:SetFormattedText(HEALTH)
  DigitsOptionsFramePower:SetFormattedText(self.strings.power)

  DigitsOptionsFrameHealthFontOutlineButtonText:SetText(self.strings.outline)
  DigitsOptionsFrameHealthPercentButtonText:SetText(self.strings.percent)
  DigitsOptionsFrameHealthOutOfCombatButtonText:SetText(self.strings.ooc)
  DigitsOptionsFrameHealthEnableButtonText:SetText(ENABLE)
  DigitsOptionsFrameHealthFontSizeLow:SetFormattedText(10)
  DigitsOptionsFrameHealthFontSizeHigh:SetFormattedText(35)
  DigitsOptionsFrameHealthFontSize:SetMinMaxValues(10, 35)
  DigitsOptionsFrameHealthFontSize:SetValueStep(1)
  DigitsOptionsFrameHealthColour.label:SetText(HEALTH)
  
  DigitsOptionsFramePowerFontOutlineButtonText:SetText(self.strings.outline)
  DigitsOptionsFramePowerPercentButtonText:SetText(self.strings.percent)
  DigitsOptionsFramePowerOutOfCombatButtonText:SetText(self.strings.ooc)
  DigitsOptionsFramePowerEnableButtonText:SetText(ENABLE)
  DigitsOptionsFramePowerFontSizeLow:SetFormattedText(10)
  DigitsOptionsFramePowerFontSizeHigh:SetFormattedText(35)
  DigitsOptionsFramePowerFontSize:SetMinMaxValues(10, 35)
  DigitsOptionsFramePowerFontSize:SetValueStep(1)
  DigitsOptionsFrameManaColour.label:SetText(MANA)
  DigitsOptionsFrameRageColour.label:SetText(RAGE)
  DigitsOptionsFrameEnergyColour.label:SetText(ENERGY)
  DigitsOptionsFrameRuneColour.label:SetText(RUNIC_POWER)
  DigitsOptionsFrameFocusColour.label:SetText(FOCUS)
  
  DigitsOptionsFrame:SetScript("OnShow", self.ShowOptions)
  DigitsOptionsFrame:SetScript("OnHide", self.CloseOptions)
  
  DigitsOptionsFrame.table = {}
  self:FontDropDown_Initialise()
end

function DigitsFrame.ShowOptions()
  DigitsFrame.showingOptions = true
  DigitsHealthFrame:EnableMouse(true)
  DigitsHealthFrame:RegisterForDrag("LeftButton")
  DigitsPowerFrame:EnableMouse(true)
  DigitsPowerFrame:RegisterForDrag("LeftButton")
  DigitsHealthFrame:SetFrameStrata("TOOLTIP")
  DigitsPowerFrame:SetFrameStrata("TOOLTIP")
  
  DigitsFrame:UpdateColours()
  DigitsFrame:UpdateHealth()
  DigitsFrame:UpdatePower()
  DigitsFrame:SetHealth()
  DigitsFrame:SetPower()
  DigitsFrame:Setup()
end

function DigitsFrame.CloseOptions()
  DigitsHealthFrame:SetFrameStrata("LOW")
  DigitsPowerFrame:SetFrameStrata("LOW")
  if ColorPickerFrame:IsShown() then
    HideUIPanel(ColorPickerFrame)
  end
  DigitsHealthFrame:EnableMouse(false)
  DigitsHealthFrame:RegisterForDrag()
  DigitsPowerFrame:EnableMouse(false)
  DigitsPowerFrame:RegisterForDrag()
  DigitsFrame.showingOptions = false
  DigitsPowerFrame.text:SetText("")
  DigitsHealthFrame.text:SetText("")
  DigitsFrame:EnablePowerFrame()
  DigitsFrame:CheckCombat()
end

function DigitsFrame:OnFont(health)
  _sdd:Open((health and DigitsOptionsFrameHealthFontButton) or DigitsOptionsFramePowerFontButton)
end

function DigitsFrame:FontDropDown_Initialise()
  local c
  for i = 1,2 do
    if (i == 1) then 
      c = DigitsOptionsFrameHealthFontButton
      DigitsOptionsFrame.table.health = true
    else 
      c = DigitsOptionsFramePowerFontButton 
      DigitsOptionsFrame.table.health = false
    end
    DigitsOptionsFrame.table.callback = self.FontDropDown_OnClick
    for k,v in pairs(_sml:List("font")) do
      DigitsOptionsFrame.table.text = v
      _sdd:AddItem(c, DigitsOptionsFrame.table)
    end
  end
end

function DigitsFrame.FontDropDown_OnClick(item)
  if item.health then
    DigitsFrame.db.healthFrame.font = item.text
    DigitsOptionsFrameHealthFontButtonLabel:SetText(item.text)
  else
    DigitsFrame.db.powerFrame.font = item.text
    DigitsOptionsFramePowerFontButtonLabel:SetText(item.text)
  end
  DigitsFrame:Setup()
end

function DigitsFrame:OnFontSize(control, health)
  if health then
    self.db.healthFrame.fontSize = control:GetValue()
    DigitsOptionsFrameHealthFontSizeText:SetFormattedText("%s (%d)", FONT_SIZE, self.db.healthFrame.fontSize)
  else
    self.db.powerFrame.fontSize = control:GetValue()
    DigitsOptionsFramePowerFontSizeText:SetFormattedText("%s (%d)", FONT_SIZE, self.db.powerFrame.fontSize)
  end
  DigitsFrame:Setup()
end

function DigitsFrame:OnOutline(checked, health)
  if health then
    self.db.healthFrame.outline = (checked and "OUTLINE") or nil
  else
    self.db.powerFrame.outline = (checked and "OUTLINE") or nil
  end
  DigitsFrame:Setup()
end

function DigitsFrame:OnPercent(checked, health)
  if health then
    self.db.healthFrame.percent = checked
    DigitsFrame:UpdateHealth()
  else
    self.db.powerFrame.percent = checked
    DigitsFrame:UpdatePower()
  end
end

function DigitsFrame:OutOfCombat(checked, health)
  if health then
    self.db.healthFrame.ooc = checked
  else
    self.db.powerFrame.ooc = checked
  end
end

function DigitsFrame:OnEnable(checked, health)
  if health then
    self.db.healthFrame.enabled = checked
  else
    self.db.powerFrame.enabled = checked
  end
  DigitsFrame:Setup()
end

function DigitsFrame:ColourPicker(index)
  if not _lcf:IsShown() then
    self.currentColour = index
    local colour = self.db[index]
    if (index < 10) then
      self.powerType = index
    end
    _lcf:Open(self.ColourCallback, colour.r, colour.g, colour.b, colour.a)
  end
end

function DigitsFrame.ColourCallback(r,g,b,a)
  DigitsFrame.db[DigitsFrame.currentColour].r = r
  DigitsFrame.db[DigitsFrame.currentColour].g = g
  DigitsFrame.db[DigitsFrame.currentColour].b = b
  DigitsFrame.db[DigitsFrame.currentColour].a = a
  DigitsFrame:Setup()
  DigitsFrame:UpdateColours()
end

function DigitsFrame:UpdateColours()
  DigitsOptionsFrameHealthColour.texture:SetVertexColor(DigitsFrame.db[10].r, DigitsFrame.db[10].g, DigitsFrame.db[10].b)
  DigitsOptionsFrameRageColour.texture:SetVertexColor(DigitsFrame.db[1].r, DigitsFrame.db[1].g, DigitsFrame.db[1].b)
  DigitsOptionsFrameManaColour.texture:SetVertexColor(DigitsFrame.db[0].r, DigitsFrame.db[0].g, DigitsFrame.db[0].b)
  DigitsOptionsFrameEnergyColour.texture:SetVertexColor(DigitsFrame.db[3].r, DigitsFrame.db[3].g, DigitsFrame.db[3].b)
  DigitsOptionsFrameRuneColour.texture:SetVertexColor(DigitsFrame.db[6].r, DigitsFrame.db[6].g, DigitsFrame.db[6].b)
  DigitsOptionsFrameFocusColour.texture:SetVertexColor(DigitsFrame.db[2].r, DigitsFrame.db[2].g, DigitsFrame.db[2].b)
end

function DigitsFrame:SetHealth()
  DigitsOptionsFrameHealthFontButtonLabel:SetText(DigitsFrame.db.healthFrame.font)
  DigitsOptionsFrameHealthFontSize:SetValue(DigitsFrame.db.healthFrame.fontSize)
  DigitsOptionsFrameHealthOutOfCombatButton:SetChecked(self.db.healthFrame.ooc)
  DigitsOptionsFrameHealthFontOutlineButton:SetChecked(DigitsFrame.db.healthFrame.outline)
  DigitsOptionsFrameHealthEnableButton:SetChecked(self.db.healthFrame.enabled)
  DigitsOptionsFrameHealthPercentButton:SetChecked(self.db.healthFrame.percent)
  _sdd:SetSelected(DigitsOptionsFrameHealthFontButton, self.db.healthFrame.font)
end

function DigitsFrame:SetPower()
  DigitsOptionsFramePowerFontButtonLabel:SetText(self.db.powerFrame.font)
  DigitsOptionsFramePowerFontSize:SetValue(self.db.powerFrame.fontSize)
  DigitsOptionsFramePowerOutOfCombatButton:SetChecked(self.db.powerFrame.ooc)
  DigitsOptionsFramePowerFontOutlineButton:SetChecked(self.db.powerFrame.outline)
  DigitsOptionsFramePowerEnableButton:SetChecked(self.db.powerFrame.enabled)
  DigitsOptionsFramePowerPercentButton:SetChecked(self.db.powerFrame.percent)
  self:DoColourButton(DigitsOptionsFrameManaColour, self.db[0].enabled)
  self:DoColourButton(DigitsOptionsFrameRageColour, self.db[1].enabled)
  self:DoColourButton(DigitsOptionsFrameEnergyColour, self.db[3].enabled)
  self:DoColourButton(DigitsOptionsFrameRuneColour, self.db[6].enabled)
  self:DoColourButton(DigitsOptionsFrameFocusColour, self.db[2].enabled)
  self:DoColourButton(DigitsOptionsFrameFocusColour, self.db[2].enabled)
  _sdd:SetSelected(DigitsOptionsFramePowerFontButton, self.db.powerFrame.font)
end

function DigitsFrame:DoColourButton(button, enabled)
  if enabled then
    button:Enable()
    button.label:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
  else
    button:Disable()
    button.label:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b)
  end
end

DigitsFrame:RegisterEvent("VARIABLES_LOADED")
DigitsFrame:SetScript("OnEvent", DigitsFrame.OnEvent)

-- Localisation
DigitsFrame.strings = {}
DigitsFrame.strings.version = "Digits |cff00ff00%.01f.%d|r"

if false and (GetLocale() == "frFR") then
  DigitsFrame.strings.outline = "Contour"
elseif false and (GetLocale() == "deDE") then
  DigitsFrame.strings.outline = "Kontur"
else
  DigitsFrame.strings.outline = DigitsFrame.strings.outline or "Outline"
  -- Needs translation
  DigitsFrame.strings.createProfile = "|cff00ff00Digits:|r Creating profile for %s"
  DigitsFrame.strings.power = "Power"
  DigitsFrame.strings.ooc = "Show out of combat"
  DigitsFrame.strings.percent = "Percentage"
end
